@@ -1,11 +1,6 @@
 """
 Context managers for use with the ``with`` statement.
 
-.. note:: When using Python 2.5, you will need to start your fabfile
-    with ``from __future__ import with_statement`` in order to make use of
-    the ``with`` statement (which is a regular, non ``__future__`` feature of
-    Python 2.6+.)
-
 .. note:: If you are using multiple directly nested ``with`` statements, it can
     be convenient to use multiple context expressions in one single with
     statement. Instead of writing::
@@ -21,26 +16,29 @@ Context managers for use with the ``with`` statement.
             run('./manage.py syncdb')
             run('./manage.py loaddata myfixture')
 
-    Note that you need Python 2.7+ for this to work. On Python 2.5 or 2.6, you
-    can do the following::
-
-        from contextlib import nested
-
-        with nested(cd('/path/to/app'), prefix('workon myvenv')):
-            ...
-
-    Finally, note that `~fabric.context_managers.settings` implements
-    ``nested`` itself -- see its API doc for details.
 """
 
-from contextlib import contextmanager, nested
-import sys
+from contextlib import contextmanager
+import six
 import socket
 import select
 
 from fabric.thread_handling import ThreadHandler
 from fabric.state import output, win32, connections, env
 from fabric import state
+from fabric.utils import isatty
+
+if six.PY2 is True:
+    from contextlib import nested
+else:
+    from contextlib import ExitStack
+
+    class nested(ExitStack):
+        def __init__(self, *managers):
+            super(nested, self).__init__()
+            for manager in managers:
+                self.enter_context(manager)
+
 
 if not win32:
     import termios
@@ -51,9 +49,9 @@ def _set_output(groups, which):
     """
     Refactored subroutine used by ``hide`` and ``show``.
     """
+    previous = {}
     try:
         # Preserve original values, pull in new given value to use
-        previous = {}
         for group in output.expand_aliases(groups):
             previous[group] = output[group]
             output[group] = which
@@ -132,7 +130,7 @@ def _setenv(variables):
     clean_revert = variables.pop('clean_revert', False)
     previous = {}
     new = []
-    for key, value in variables.iteritems():
+    for key, value in six.iteritems(variables):
         if key in state.env:
             previous[key] = state.env[key]
         else:
@@ -142,7 +140,7 @@ def _setenv(variables):
         yield
     finally:
         if clean_revert:
-            for key, value in variables.iteritems():
+            for key, value in six.iteritems(variables):
                 # If the current env value for this key still matches the
                 # value we set it to beforehand, we are OK to revert it to the
                 # pre-block value.
@@ -328,8 +326,8 @@ def lcd(path):
 
 
 def _change_cwd(which, path):
-    path = path.replace(' ', '\ ')
-    if state.env.get(which) and not path.startswith('/'):
+    path = path.replace(' ', r'\ ')
+    if state.env.get(which) and not path.startswith('/') and not path.startswith('~'):
         new_cwd = state.env.get(which) + '/' + path
     else:
         new_cwd = path
@@ -429,7 +427,7 @@ def char_buffered(pipe):
 
     Only applies on Unix-based systems; on Windows this is a no-op.
     """
-    if win32 or not pipe.isatty():
+    if win32 or not isatty(pipe):
         yield
     else:
         old_settings = termios.tcgetattr(pipe)
@@ -522,6 +520,8 @@ def remote_tunnel(remote_port, local_port=None, local_host="localhost",
         By default, most SSH servers only allow remote tunnels to listen to the
         localhost interface (127.0.0.1). In these cases, `remote_bind_address`
         is ignored by the server, and the tunnel will listen only to 127.0.0.1.
+
+    .. versionadded: 1.6
     """
     if local_port is None:
         local_port = remote_port
@@ -530,21 +530,24 @@ def remote_tunnel(remote_port, local_port=None, local_host="localhost",
     channels = []
     threads = []
 
-    def accept(channel, (src_addr, src_port), (dest_addr, dest_port)):
+    def accept(channel, src, dest):
+        src_addr, src_port = src
+        dest_addr, dest_port = dest
         channels.append(channel)
         sock = socket.socket()
         sockets.append(sock)
 
         try:
             sock.connect((local_host, local_port))
-        except Exception, e:
-            print "[%s] rtunnel: cannot connect to %s:%d (from local)" % (env.host_string, local_host, local_port)
-            chan.close()
+        except Exception:
+            print("[%s] rtunnel: cannot connect to %s:%d (from local)" %
+                  (env.host_string, local_host, local_port))
+            channel.close()
             return
 
-        print "[%s] rtunnel: opened reverse tunnel: %r -> %r -> %r"\
+        print("[%s] rtunnel: opened reverse tunnel: %r -> %r -> %r"
               % (env.host_string, channel.origin_addr,
-                 channel.getpeername(), (local_host, local_port))
+                 channel.getpeername(), (local_host, local_port)))
 
         th = ThreadHandler('fwd', _forwarder, channel, sock)
         threads.append(th)
@@ -563,7 +566,6 @@ def remote_tunnel(remote_port, local_port=None, local_host="localhost",
         transport.cancel_port_forward(remote_bind_address, remote_port)
 
 
-
 quiet = lambda: settings(hide('everything'), warn_only=True)
 quiet.__doc__ = """
     Alias to ``settings(hide('everything'), warn_only=True)``.
