@@ -5,14 +5,17 @@ Functions to be used in fabfiles and other non-core code, such as run()/sudo().
 
 from __future__ import with_statement
 
+import errno
 import os
 import os.path
 import posixpath
 import re
+import six
 import subprocess
 import sys
 import time
 from glob import glob
+from collections import deque
 from contextlib import closing, contextmanager
 
 from fabric.context_managers import (settings, char_buffered, hide,
@@ -23,13 +26,7 @@ from fabric.sftp import SFTP
 from fabric.state import env, connections, output, win32, default_channel
 from fabric.thread_handling import ThreadHandler
 from fabric.utils import (
-    abort,
-    error,
-    handle_prompt_abort,
-    indent,
-    _pty_size,
-    warn,
-    apply_lcwd
+    abort, error, handle_prompt_abort, indent, _pty_size, warn, apply_lcwd,
 )
 
 
@@ -98,6 +95,7 @@ def require(*keys, **kwargs):
     # If all keys exist and are non-empty, we're good, so keep going.
     missing_keys = filter(lambda x: x not in env or (x in env and
         isinstance(env[x], (dict, list, tuple, set)) and not env[x]), keys)
+    missing_keys = list(missing_keys)
     if not missing_keys:
         return
     # Pluralization
@@ -212,7 +210,10 @@ def prompt(text, key=None, default='', validate=None):
     value = None
     while value is None:
         # Get input
-        value = raw_input(prompt_str) or default
+        if six.PY3 is True:
+            value = input(prompt_str) or default
+        else:
+            value = raw_input(prompt_str) or default
         # Handle validation
         if validate:
             # Callable
@@ -221,7 +222,7 @@ def prompt(text, key=None, default='', validate=None):
                 # fails.
                 try:
                     value = validate(value)
-                except Exception, e:
+                except Exception as e:
                     # Reset value so we stay in the loop
                     value = None
                     print("Validation failed for the following reason:")
@@ -256,6 +257,9 @@ def put(local_path=None, remote_path=None, use_sudo=False,
     """
     Upload one or more files to a remote host.
 
+    As with the OpenSSH ``sftp`` program, `.put` will overwrite pre-existing
+    remote files without requesting confirmation.
+
     `~fabric.operations.put` returns an iterable containing the absolute file
     paths of all remote files uploaded. This iterable also exhibits a
     ``.failed`` attribute containing any local file paths which failed to
@@ -295,7 +299,8 @@ def put(local_path=None, remote_path=None, use_sudo=False,
     scripts). To do this, specify ``mirror_local_mode=True``.
 
     Alternately, you may use the ``mode`` kwarg to specify an exact mode, in
-    the same vein as ``os.chmod`` or the Unix ``chmod`` command.
+    the same vein as ``os.chmod``, such as an exact octal number (``0o755``) or
+    a string representing one (``"0o755"``).
 
     `~fabric.operations.put` will honor `~fabric.context_managers.cd`, so
     relative values in ``remote_path`` will be prepended by the current remote
@@ -313,7 +318,7 @@ def put(local_path=None, remote_path=None, use_sudo=False,
 
         put('bin/project.zip', '/tmp/project.zip')
         put('*.py', 'cgi-bin/')
-        put('index.html', 'index.html', mode=0755)
+        put('index.html', 'index.html', mode=0o755)
 
     .. note::
         If a file-like object such as StringIO has a ``name`` attribute, that
@@ -398,7 +403,7 @@ def put(local_path=None, remote_path=None, use_sudo=False,
                     p = ftp.put(lpath, remote_path, use_sudo, mirror_local_mode,
                         mode, local_is_path, temp_dir)
                     remote_paths.append(p)
-            except Exception, e:
+            except Exception as e:
                 msg = "put() encountered an exception while uploading '%s'"
                 failure = lpath if local_is_path else "<StringIO>"
                 failed_local_paths.append(failure)
@@ -411,7 +416,7 @@ def put(local_path=None, remote_path=None, use_sudo=False,
 
 
 @needs_host
-def get(remote_path, local_path=None):
+def get(remote_path, local_path=None, use_sudo=False, temp_dir=""):
     """
     Download one or more files from a remote host.
 
@@ -445,6 +450,13 @@ def get(remote_path, local_path=None):
       ``utils.py`` in ``src/projectname/utils.py``
     * ``path``: The full remote path, e.g. ``src/projectname/utils.py``.
 
+    While the SFTP protocol (which `get` uses) has no direct ability to download
+    files from locations not owned by the connecting user, you may specify
+    ``use_sudo=True`` to work around this. When set, this setting allows `get`
+    to copy (using sudo) the remote files to a temporary location on the remote end
+    (defaults to remote user's ``$HOME``; this may be overridden via ``temp_dir``),
+    and then download them to ``local_path``.
+
     .. note::
         When ``remote_path`` is an absolute directory path, only the inner
         directories will be recreated locally and passed into the above
@@ -552,12 +564,13 @@ def get(remote_path, local_path=None):
         failed_remote_files = []
 
         try:
-            # Glob remote path if it's a directory; otherwise use as-is
-            if (
-                ftp.isdir(remote_path)
-                or '*' in remote_path or '?' in remote_path
-            ):
+            # Glob remote path if necessary
+            if '*' in remote_path or '?' in remote_path:
                 names = ftp.glob(remote_path)
+                # Handle "file not found" errors (like Paramiko does if we
+                # explicitly try to grab a glob-like filename).
+                if not names:
+                    raise IOError(errno.ENOENT, "No such file")
             else:
                 names = [remote_path]
 
@@ -568,18 +581,17 @@ def get(remote_path, local_path=None):
 
             for remote_path in names:
                 if ftp.isdir(remote_path):
-                    result = ftp.get_dir(remote_path, local_path)
+                    result = ftp.get_dir(remote_path, local_path, use_sudo, temp_dir)
                     local_files.extend(result)
                 else:
                     # Perform actual get. If getting to real local file path,
                     # add result (will be true final path value) to
                     # local_files. File-like objects are omitted.
-                    result = ftp.get(remote_path, local_path, local_is_path,
-                        os.path.basename(remote_path))
+                    result = ftp.get(remote_path, local_path, use_sudo, local_is_path, os.path.basename(remote_path), temp_dir)
                     if local_is_path:
                         local_files.append(result)
 
-        except Exception, e:
+        except Exception as e:
             failed_remote_files.append(remote_path)
             msg = "get() encountered an exception while downloading '%s'"
             error(message=msg % remote_path, exception=e)
@@ -657,8 +669,9 @@ def _prefix_commands(command, which):
     # Also place it at the front of the list, in case user is expecting another
     # prefixed command to be "in" the current working directory.
     cwd = env.cwd if which == 'remote' else env.lcwd
+    redirect = " >/dev/null" if not win32 else ''
     if cwd:
-        prefixes.insert(0, 'cd %s' % cwd)
+        prefixes.insert(0, 'cd %s%s' % (cwd, redirect))
     glue = " && "
     prefix = (glue.join(prefixes) + glue) if prefixes else ""
     return prefix + command
@@ -701,7 +714,7 @@ def _prefix_env_vars(command, local=False):
 
         exports = ' '.join(
             '%s%s="%s"' % (set_cmd, k, v if k == 'PATH' else _shell_escape(v))
-            for k, v in env_vars.iteritems()
+            for k, v in six.iteritems(env_vars)
         )
         shell_env_str = '%s%s && ' % (exp_cmd, exports)
     else:
@@ -711,7 +724,8 @@ def _prefix_env_vars(command, local=False):
 
 
 def _execute(channel, command, pty=True, combine_stderr=None,
-    invoke_shell=False, stdout=None, stderr=None, timeout=None):
+    invoke_shell=False, stdout=None, stderr=None, timeout=None,
+    capture_buffer_size=None):
     """
     Execute ``command`` over ``channel``.
 
@@ -726,6 +740,10 @@ def _execute(channel, command, pty=True, combine_stderr=None,
     ``invoke_shell`` (plus a handful of other things, such as always forcing a
     pty.)
 
+    ``capture_buffer_size`` controls the length of the ring-buffers used to
+    capture stdout/stderr. (This is ignored if ``invoke_shell=True``, since
+    that completely disables capturing overall.)
+
     Returns a three-tuple of (``stdout``, ``stderr``, ``status``), where
     ``stdout``/``stderr`` are captured output strings and ``status`` is the
     program's return code, if applicable.
@@ -773,7 +791,8 @@ def _execute(channel, command, pty=True, combine_stderr=None,
 
         # Init stdout, stderr capturing. Must use lists instead of strings as
         # strings are immutable and we're using these as pass-by-reference
-        stdout_buf, stderr_buf = [], []
+        stdout_buf = deque(maxlen=capture_buffer_size)
+        stderr_buf = deque(maxlen=capture_buffer_size)
         if invoke_shell:
             stdout_buf = stderr_buf = None
 
@@ -878,7 +897,8 @@ def _noop():
 
 def _run_command(command, shell=True, pty=True, combine_stderr=True,
     sudo=False, user=None, quiet=False, warn_only=False, stdout=None,
-    stderr=None, group=None, timeout=None, shell_escape=None):
+    stderr=None, group=None, timeout=None, shell_escape=None,
+    capture_buffer_size=None):
     """
     Underpinnings of `run` and `sudo`. See their docstrings for more info.
     """
@@ -898,7 +918,7 @@ def _run_command(command, shell=True, pty=True, combine_stderr=True,
 
         # Handle context manager modifications, and shell wrapping
         wrapped_command = _shell_wrap(
-            _prefix_commands(_prefix_env_vars(command), 'remote'),
+            _prefix_env_vars(_prefix_commands(command, 'remote')),
             shell_escape,
             shell,
             _sudo_prefix(user, group) if sudo else None
@@ -914,7 +934,8 @@ def _run_command(command, shell=True, pty=True, combine_stderr=True,
         result_stdout, result_stderr, status = _execute(
             channel=default_channel(), command=wrapped_command, pty=pty,
             combine_stderr=combine_stderr, invoke_shell=False, stdout=stdout,
-            stderr=stderr, timeout=timeout)
+            stderr=stderr, timeout=timeout,
+            capture_buffer_size=capture_buffer_size)
 
         # Assemble output string
         out = _AttributeString(result_stdout)
@@ -952,7 +973,8 @@ def _run_command(command, shell=True, pty=True, combine_stderr=True,
 
 @needs_host
 def run(command, shell=True, pty=True, combine_stderr=None, quiet=False,
-    warn_only=False, stdout=None, stderr=None, timeout=None, shell_escape=None):
+    warn_only=False, stdout=None, stderr=None, timeout=None, shell_escape=None,
+    capture_buffer_size=None):
     """
     Run a shell command on a remote host.
 
@@ -960,7 +982,13 @@ def run(command, shell=True, pty=True, combine_stderr=None, quiet=False,
     string via a shell interpreter, the value of which may be controlled by
     setting ``env.shell`` (defaulting to something similar to ``/bin/bash -l -c
     "<command>"``.) Any double-quote (``"``) or dollar-sign (``$``) characters
-    in ``command`` will be automatically escaped when ``shell`` is True.
+    in ``command`` will be automatically escaped when ``shell`` is True (unless
+    disabled by setting ``shell_escape=False``).
+
+    When ``shell=False``, no shell wrapping or escaping will occur. (It's
+    possible to specify ``shell=False, shell_escape=True`` if desired, which
+    will still trigger escaping of dollar signs, etc but will not wrap with a
+    shell program invocation).
 
     `run` will return the result of the remote program's stdout as a single
     (likely multiline) string. This string will exhibit ``failed`` and
@@ -970,6 +998,21 @@ def run(command, shell=True, pty=True, combine_stderr=None, quiet=False,
     command strings executed, as ``.command`` and ``.real_command``,
     respectively.
 
+    To lessen memory use when running extremely verbose programs (and,
+    naturally, when having access to their full output afterwards is not
+    necessary!) you may limit how much of the program's stdout/err is stored by
+    setting ``capture_buffer_size`` to an integer value.
+
+    .. warning::
+        Do not set ``capture_buffer_size`` to any value smaller than the length
+        of ``env.sudo_prompt`` or you will likely break the functionality of
+        `sudo`! Ditto any user prompts stored in ``env.prompts``.
+
+    .. note::
+        This value is used for each buffer independently, so e.g. ``1024`` may
+        result in storing a total of ``2048`` bytes if there's data in both
+        streams.)
+
     Any text entered in your local terminal will be forwarded to the remote
     program as it runs, thus allowing you to interact with password or other
     prompts naturally. For more on how this works, see
@@ -1042,16 +1085,21 @@ def run(command, shell=True, pty=True, combine_stderr=None, quiet=False,
 
     .. versionadded:: 1.7
         The ``shell_escape`` argument.
+
+    .. versionadded:: 1.11
+        The ``capture_buffer_size`` argument.
     """
-    return _run_command(command, shell, pty, combine_stderr, quiet=quiet,
+    return _run_command(
+        command, shell, pty, combine_stderr, quiet=quiet,
         warn_only=warn_only, stdout=stdout, stderr=stderr, timeout=timeout,
-        shell_escape=shell_escape)
+        shell_escape=shell_escape, capture_buffer_size=capture_buffer_size,
+    )
 
 
 @needs_host
 def sudo(command, shell=True, pty=True, combine_stderr=None, user=None,
     quiet=False, warn_only=False, stdout=None, stderr=None, group=None,
-    timeout=None, shell_escape=None):
+    timeout=None, shell_escape=None, capture_buffer_size=None):
     """
     Run a shell command on a remote host, with superuser privileges.
 
@@ -1093,12 +1141,16 @@ def sudo(command, shell=True, pty=True, combine_stderr=None, user=None,
 
     .. versionadded:: 1.7
         The ``shell_escape`` argument.
+
+    .. versionadded:: 1.11
+        The ``capture_buffer_size`` argument.
     """
     return _run_command(
         command, shell, pty, combine_stderr, sudo=True,
         user=user if user else env.sudo_user,
         group=group, quiet=quiet, warn_only=warn_only, stdout=stdout,
         stderr=stderr, timeout=timeout, shell_escape=shell_escape,
+        capture_buffer_size=capture_buffer_size,
     )
 
 
@@ -1171,19 +1223,20 @@ def local(command, capture=False, shell=None):
         err_stream = None if output.stderr else dev_null
     try:
         cmd_arg = wrapped_command if win32 else [wrapped_command]
-        if shell is not None:
-            p = subprocess.Popen(cmd_arg, shell=True, stdout=out_stream,
-                                 stderr=err_stream, executable=shell)
-        else:
-            p = subprocess.Popen(cmd_arg, shell=True, stdout=out_stream,
-                                 stderr=err_stream)
+        p = subprocess.Popen(cmd_arg, shell=True, stdout=out_stream,
+                             stderr=err_stream, executable=shell,
+                             close_fds=(not win32))
         (stdout, stderr) = p.communicate()
     finally:
         if dev_null is not None:
             dev_null.close()
     # Handle error condition (deal with stdout being None, too)
-    out = _AttributeString(stdout.strip() if stdout else "")
-    err = _AttributeString(stderr.strip() if stderr else "")
+    if six.PY3:
+        out = _AttributeString(stdout.decode('utf-8').strip() if stdout else "")
+        err = _AttributeString(stderr.decode('utf-8').strip() if stderr else "")
+    else:
+        out = _AttributeString(stdout.strip() if stdout else "")
+        err = _AttributeString(stderr.strip() if stderr else "")
     out.command = given_command
     out.real_command = wrapped_command
     out.failed = False
@@ -1199,7 +1252,7 @@ def local(command, capture=False, shell=None):
 
 
 @needs_host
-def reboot(wait=120, command='reboot'):
+def reboot(wait=120, command='reboot', use_sudo=True):
     """
     Reboot the remote system.
 
@@ -1223,6 +1276,9 @@ def reboot(wait=120, command='reboot'):
         Changed the ``wait`` kwarg to be optional, and refactored to leverage
         the new reconnection functionality; it may not actually have to wait
         for ``wait`` seconds before reconnecting.
+    .. versionchanged:: 1.11
+        Added ``use_sudo`` as a kwarg. Maintained old functionality by setting
+        the default value to True.
     """
     # Shorter timeout for a more granular cycle than the default.
     timeout = 5
@@ -1236,7 +1292,7 @@ def reboot(wait=120, command='reboot'):
         timeout=timeout,
         connection_attempts=attempts
     ):
-        sudo(command)
+        (sudo if use_sudo else run)(command)
         # Try to make sure we don't slip in before pre-reboot lockdown
         time.sleep(5)
         # This is actually an internal-ish API call, but users can simply drop
