@@ -3,17 +3,40 @@ Internal subroutines for e.g. aborting execution with an error message,
 or performing indenting on multiline output.
 """
 import os
+import six
 import sys
 import textwrap
 from traceback import format_exc
 
+
+def _encode(msg, stream):
+    if six.PY2 and isinstance(msg, unicode) and hasattr(stream, 'encoding') and not stream.encoding is None:
+        return msg.encode(stream.encoding)
+    else:
+        return str(msg)
+
+
+def isatty(stream):
+    """Check if a stream is a tty.
+
+    Not all file-like objects implement the `isatty` method.
+    """
+    fn = getattr(stream, 'isatty', None)
+    if fn is None:
+        return False
+    return fn()
+
+
 def abort(msg):
     """
     Abort execution, print ``msg`` to stderr and exit with error status (1.)
 
-    This function currently makes use of `sys.exit`_, which raises
-    `SystemExit`_. Therefore, it's possible to detect and recover from inner
-    calls to `abort` by using ``except SystemExit`` or similar.
+    This function currently makes use of `SystemExit`_ in a manner that is
+    similar to `sys.exit`_ (but which skips the automatic printing to stderr,
+    allowing us to more tightly control it via settings).
+
+    Therefore, it's possible to detect and recover from inner calls to `abort`
+    by using ``except SystemExit`` or similar.
 
     .. _sys.exit: http://docs.python.org/library/sys.html#sys.exit
     .. _SystemExit: http://docs.python.org/library/exceptions.html#exceptions.SystemExit
@@ -22,16 +45,22 @@ def abort(msg):
     if not env.colorize_errors:
         red  = lambda x: x
     else:
-        from colors import red
+        from fabric.colors import red
 
     if output.aborts:
-        sys.stderr.write(red("\nFatal error: %s\n" % str(msg)))
+        sys.stderr.write(red("\nFatal error: %s\n" % _encode(msg, sys.stderr)))
         sys.stderr.write(red("\nAborting.\n"))
 
     if env.abort_exception:
         raise env.abort_exception(msg)
     else:
-        sys.exit(1)
+        # See issue #1318 for details on the below; it lets us construct a
+        # valid, useful SystemExit while sidestepping the automatic stderr
+        # print (which would otherwise duplicate with the above in a
+        # non-controllable fashion).
+        e = SystemExit(1)
+        e.message = msg
+        raise e
 
 
 def warn(msg):
@@ -48,9 +77,10 @@ def warn(msg):
     if not env.colorize_errors:
         magenta = lambda x: x
     else:
-        from colors import magenta
+        from fabric.colors import magenta
 
     if output.warnings:
+        msg = _encode(msg, sys.stderr)
         sys.stderr.write(magenta("\nWarning: %s\n\n" % msg))
 
 
@@ -110,7 +140,7 @@ def puts(text, show_prefix=None, end="\n", flush=False):
         prefix = ""
         if env.host_string and show_prefix:
             prefix = "[%s] " % env.host_string
-        sys.stdout.write(prefix + str(text) + end)
+        sys.stdout.write(prefix + _encode(text, sys.stdout) + end)
         if flush:
             sys.stdout.flush()
 
@@ -265,7 +295,7 @@ def _pty_size():
 
     default_rows, default_cols = 24, 80
     rows, cols = default_rows, default_cols
-    if not win32 and sys.stdout.isatty():
+    if not win32 and isatty(sys.stdout):
         # We want two short unsigned integers (rows, cols)
         fmt = 'HH'
         # Create an empty (zeroed) buffer for ioctl to map onto. Yay for C!
@@ -305,9 +335,11 @@ def error(message, func=None, exception=None, stdout=None, stderr=None):
     import fabric.state
     if func is None:
         func = fabric.state.env.warn_only and warn or abort
-    # If debug printing is on, append a traceback to the message
-    if fabric.state.output.debug:
-        message += "\n\n" + format_exc()
+    # If exception printing is on, append a traceback to the message
+    if fabric.state.output.exceptions or fabric.state.output.debug:
+        exception_message = format_exc()
+        if exception_message:
+            message += "\n\n" + exception_message
     # Otherwise, if we were given an exception, append its contents.
     elif exception is not None:
         # Figure out how to get a string out of the exception; EnvironmentError
@@ -329,7 +361,7 @@ def error(message, func=None, exception=None, stdout=None, stderr=None):
 
 def _format_error_output(header, body):
     term_width = _pty_size()[1]
-    header_side_length = (term_width - (len(header) + 2)) / 2
+    header_side_length = int((term_width - (len(header) + 2)) / 2)
     mark = "="
     side = mark * header_side_length
     return "\n\n%s %s %s\n\n%s\n\n%s" % (
@@ -337,42 +369,6 @@ def _format_error_output(header, body):
     )
 
 
-# TODO: replace with collections.deque(maxlen=xxx) in Python 2.6
-class RingBuffer(list):
-    def __init__(self, value, maxlen):
-        # Heh.
-        self._super = super(RingBuffer, self)
-        self._maxlen = maxlen
-        return self._super.__init__(value)
-
-    def _free(self):
-        return self._maxlen - len(self)
-
-    def append(self, value):
-        if self._free() == 0:
-            del self[0]
-        return self._super.append(value)
-
-    def extend(self, values):
-        overage = len(values) - self._free()
-        if overage > 0:
-            del self[0:overage]
-        return self._super.extend(values)
-
-    # Paranoia from here on out.
-    def insert(self, index, value):
-        raise ValueError("Can't insert into the middle of a ring buffer!")
-
-    def __setslice__(self, i, j, sequence):
-        raise ValueError("Can't set a slice of a ring buffer!")
-
-    def __setitem__(self, key, value):
-        if isinstance(key, slice):
-            raise ValueError("Can't set a slice of a ring buffer!")
-        else:
-            return self._super.__setitem__(key, value)
-
-
 def apply_lcwd(path, env):
     # Apply CWD if a relative path
     if not os.path.isabs(path) and env.lcwd:
