@@ -1,5 +1,3 @@
-from __future__ import with_statement
-
 import random
 import sys
 
@@ -13,7 +11,7 @@ import fabric # for patching fabric.state.xxx
 from fabric.tasks import _parallel_tasks, requires_parallel, execute
 from fabric.context_managers import lcd, settings, hide
 
-from utils import mock_streams
+from mock_streams import mock_streams
 
 
 #
@@ -222,7 +220,7 @@ def test_global_parallel_honors_runs_once():
     """
     @decorators.runs_once
     def mytask():
-        print "yolo" # 'Carpe diem' for stupid people!
+        print("yolo") # 'Carpe diem' for stupid people!
     with settings(hide('everything'), parallel=True):
         execute(mytask, hosts=['localhost', '127.0.0.1'])
     result = sys.stdout.getvalue()
